#include "gfx.hh"
#include "error.hh"

#include "gfx_api.hh"
#include "gfx_term.hh"
#include "gfx_gfx.hh"

static Screen* CurScreen = 0;
static int CurScreenMode = 0;

void GFX_Init()
{
    Term_Init();
    CurScreen = new TerminalScreen;
}
void GFX_Done()
{
    delete CurScreen;
    CurScreen = 0;
    Term_Done();
}

int B_FuncCSRLIN()
{
    std::pair<int,int> pos = CurScreen->GetCursorPos();
    return pos.second;
}

int B_FuncPOS(int)
{
    std::pair<int,int> pos = CurScreen->GetCursorPos();
    return pos.first;
}

int B_FuncWIDTH()
{
    std::pair<int,int> pos = CurScreen->GetWinSize();
    return pos.first;
}

void GFX_OutS(const char* buffer, size_t nbytes)
{
    CurScreen->OutS(buffer, nbytes);
}

void B_StmtVIEWPRINT(int from, int to)
{
    CurScreen->SetViewPrint(from, to);
}

int B_FuncSCREEN(int y,int x, int colorflag)
{
    std::pair<int,int> chr = CurScreen->ReadScreen(x,y);
    if(colorflag)
        return chr.second;
    return chr.first;
}

void B_StmtCLS(int type)
{
    CurScreen->CLS(type);
}

void B_StmtWIDTH(int w,int h)
{
    CurScreen->SetWinSize(std::pair<int,int>(w,h), false);
}

void B_StmtSCREEN(int mode,int color,int apage,int vpage)
{
    color=color; // We ignore the color parameter.
    
    /*
     SCREEN GWIDTH TWIDTH   ATTRIBUTES  PALETTE   PAGES
     0              40x25   16,16,16    64        8
     0              40x43   16,16,16    64        8
     0              40x50   16,16,16    64        8
     0              80x25   16,16,16    64        8
     0              80x43   16,16,16    64        8
     0              80x50   16,16,16    64        8
     
     1      320x200 40x25   4,4         16        1
     2      640x200 80x25   2,2         16        1
     3      720x348 80x25   1,1         -         2
     4      640x400 80x25   16,1        -         1
     7      320x200 40x25   16,16       16        8
     8      640x200 80x25   16,16       16        4
     
     9      640x350 80x25   16,16       64        2
     9      640x350 80x43   16,16       64        2
     
     10     640x350 80x25   4,4         9         2
     10     640x350 80x43   4,4         9         2
     
     11     640x480 80x30   2,2         262144    1
     11     640x480 80x60   2,2         262144    1
     
     12     640x480 80x30   16,16       262144    1
     12     640x480 80x60   16,16       262144    1
     
     13     320x200 40x25   256,256     262144    1
    */
    
    std::pair<int,int> oldwidth = CurScreen->GetWinSize();
    
    if(mode == 0)
    {
        if(!dynamic_cast<TerminalScreen*>(CurScreen))
            { delete CurScreen;
              CurScreen = new TerminalScreen;
              CurScreen->SetPages(8);
            }
    }
    else
    {
        int w=0, h=0; // text dimensions
        int xs=0, ys=0;   // gfx dimensions
        int fc=0, bc=0;   // default colors
        int cm=0, pm=0;   // color mode, palette mode
        int p=0; // pages
        
        switch(mode)
        {
            case 1:
                // pal: 0x00,0x3B,0x3D,0x3F
                w=40; h=25; xs=320; ys=200; fc=3; bc=0; p=1; break;
            case 2:
                // pal: 0x00,0x3F
                w=80; h=25; xs=640; ys=200; fc=1; bc=0; p=1; break;
            case 3:
                w=80; h=25; xs=720; ys=348; fc=1; bc=0; p=2; break;
            case 4:
                w=80; h=25; xs=640; ys=400; fc=1; bc=0; p=1; break;
            case 7:
                w=40; h=25; xs=320; ys=200; fc=15;bc=0; p=8; break;
            case 8:
                w=80; h=25; xs=640; ys=200; fc=15;bc=0; p=4; break;
            case 9:
                // color mapping: 0123 -> 0145
                w=80; h=25; xs=640; ys=350; fc=15;bc=0; p=2; break;
            case 10:
                // pal: 0,8,0,0,24,24,0,0,0,8,0,0,0,24,0,0
                // color mapping: 0123 -> 0145
                w=80; h=25; xs=640; ys=350; fc=3; bc=0; p=2; break;
            case 11:
                // pal: 0,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,0
                w=80; h=30; xs=640; ys=480; fc=1; bc=0; p=1; break;
            case 12:
                // pal: 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16
                w=80; h=30; xs=640; ys=480; fc=15;bc=0; p=1; break;
            case 13:
                // pal: 256 colors
                w=40; h=25; xs=320; ys=200; fc=15;bc=0; p=1; break;
            default:
                B_ERROR(5); // Illegal function call
        }
        
        /*
        GraphicScreen* gfx = dynamic_cast<GraphicScreen*>(CurScreen);
        if(!gfx) { delete CurScreen; CurScreen = gfx = new GraphicScreen; }
        
        gfx->SetGFXresolution(xs,ys, p);
        gfx->SetWinSize(std::pair<int,int>(w,h), false);
        gfx->SetColor(fc,bc);
        */
    }
    // SCREEN 0: 40 x 25, 40 x 43, 40 x 50, 80 x 25, 80 x 43, or 80 x 50 text
    // 64 colors assigned to any of 16 attributes (with EGA or VGA)
    // Depending on the text resolution and adapter, 8 video memory pages
    //      (0-7), 4 pages (0-3), 2 pages (0-1), or 1 page (0)
    
    // SCREEN 1: 320x200 gfx, 40x25
    
    CurScreen->SetWinSize(oldwidth, true); // try to approximate the same size
    if(apage >= 0) CurScreen->SetAPage(apage);
    if(vpage >= 0) CurScreen->SetVPage(vpage);
}

void B_StmtCOLOR(int fg,int bg,int border)
{
    if(CurScreenMode == 1)
    {
        // In screen 1, the following exceptions apply:
        // fg means background
        // bg means palette
        CurScreen->SetColor(0, fg, border);
        if(bg >= 0) switch(bg & 1)
        {
            case 0:
                // set the following palettes:
                //   1 = 2(green); 2 = 4(red); 3 = 6(yellow)
                //FIXME
                break;
            case 1:
                // set the following palettes:
                //   1 = 3(cyan); 2 = 5(magenta); 3=7(white)
                //FIXME
                break;
        }
        return;
    }
    
    CurScreen->SetColor(fg, bg, border);
}

void B_StmtLOCATE(int row,int col,int c_vis,int c_start,int c_stop)
{
    if(row != -1 || col != -1)
    {
        std::pair<int,int> pos;
        if(row == -1 || col == -1) pos = CurScreen->GetCursorPos();
        if(col != -1) pos.first = col;
        if(row != -1) pos.second = row;
        CurScreen->SetCursorPos(pos);
    }
    if(c_vis != -1)
        CurScreen->SetCursorVisibility(c_vis);
    if(c_start != -1 || c_stop != -1)
        CurScreen->SetCursorSize(c_start, c_stop);
}

void B_StmtLOCATE(int row,int col,int c_vis)
{
    B_StmtLOCATE(row,col,c_vis,-1,-1);
}

void B_StmtLOCATE(int row,int col)
{
    B_StmtLOCATE(row,col,-1);
}

void B_ScreenPOKE(int x,int y, int colorflag, int value)
{
    CurScreen->ScreenPOKE(x,y,colorflag,value);
}
